@@ -14,6 +14,7 @@ Options:
   --refresh=<seconds>    Refresh display every <seconds> [default: 1].
   --columns=<columns>    List of optional columns to display. Always includes 'name'. [default: owner,processes,memory,cpu-sys,cpu-user,blkio,cpu-time].
   --sort-col=<sort-col>  Select column to sort by initially. Can be changed dynamically. [default: cpu-user]
+  --type=[types]         Only keep containers of this types
   -h --help              Show this screen.
 
 '''
@@ -26,14 +27,17 @@ import stat
 import pwd
 import time
 import pty
+import errno
 import subprocess
 import multiprocessing
+import json
 
 from collections import defaultdict
 from collections import namedtuple
 
 from optparse import OptionParser
 
+
 try:
     import curses, _curses
 except ImportError:
@@ -41,10 +45,16 @@ except ImportError:
     sys.exit(0)
 
 def cmd_exists(cmd):
-    return subprocess.call(["/bin/which",  cmd], stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0
+    try:
+        return subprocess.call(["which",  cmd], stdout=subprocess.PIPE, stderr=subprocess.PIPE) == 0
+    except OSError:
+        return False
 
 HAS_LXC = cmd_exists('lxc-start')
 HAS_DOCKER = cmd_exists('docker')
+HAS_OPENVZ = cmd_exists('vzctl')
+regexp_ovz_container = re.compile('^/\d+$')
+
 
 HIDE_EMPTY_CGROUP = True
 CGROUP_MOUNTPOINTS={}
@@ -57,10 +67,12 @@ CONFIGURATION = {
         'refresh_interval': 1.0,
         'columns': [],
         'selected_line': None,
+        'offset': 0,
         'selected_line_num': 0,
         'selected_line_name': '/',
         'cgroups': [],
         'fold': [],
+        'type': [],
 }
 
 Column = namedtuple('Column', ['title', 'width', 'align', 'col_fmt', 'col_data', 'col_sort'])
@@ -70,7 +82,7 @@ COLUMNS_MANDATORY = ['name']
 COLUMNS_AVAILABLE = {
     'owner':     Column("OWNER",   10, '<', '{0:%ss}',      'owner',           'owner'),
     'type':      Column("TYPE",    10, '<', '{0:%ss}',      'type',            'type'),
-    'processes': Column("PROC",     4, '>', '{0:%sd}',      'tasks',           'tasks'),
+    'processes': Column("PROC",    11, '>', '{0:%ss}',      'tasks',           'tasks'),
     'memory':    Column("MEMORY",  17, '^', '{0:%ss}',      'memory_cur_str',  'memory_cur_bytes'),
     'cpu-sys':   Column("SYST",     5, '^', '{0: >%s.1%%}', 'cpu_syst',        'cpu_total'),
     'cpu-user':  Column("USER",     5, '^', '{0: >%s.1%%}', 'cpu_user',        'cpu_total'),
@@ -79,6 +91,8 @@ COLUMNS_AVAILABLE = {
     'name':      Column("CGROUP",  '', '<', '{0:%ss}',      'cgroup',          'cgroup'),
 }
 
+DOCKER_PREFIXES = ["/docker/", "/system.slice/docker-", "/system.slice/docker/"]
+
 # TODO:
 # - visual CPU/memory usage
 # - auto-color
@@ -89,6 +103,60 @@ COLUMNS_AVAILABLE = {
 
 ## Utils
 
+
+def strip_prefix(prefix, text):
+    if text.startswith(prefix):
+        return text[len(prefix):]
+    return text
+
+
+def docker_container_name(container_id, default, cache=dict()):
+    # Python's default arguments are evaluated when the function is
+    # defined, not when the function is called.
+    # We potentially cache and return a default value so we don't spend time
+    # pointlessly retrying to get the container name if something goes wrong.
+    cached_name = cache.get(container_id)
+    if cached_name:
+        return cached_name
+
+    try:
+        sp = subprocess.Popen(['docker', 'inspect', container_id],
+                              stdout=subprocess.PIPE,
+                              stderr=subprocess.PIPE)
+    except OSError:
+        # `docker` is not on PATH
+        cache[container_id] = default
+        return default
+
+    for _ in range(10):
+        sp.poll()
+        if sp.returncode is not None:
+            break
+        time.sleep(0.1)
+
+    if sp.returncode is None:
+        try:
+            sp.kill()
+        except OSError:
+            # OSError: [Errno 3] No such process
+            pass
+    elif sp.returncode == 0:
+        stdout, _stderr = sp.communicate()
+        try:
+            containers = json.loads(stdout)
+            if len(containers) == 1:
+                container = containers[0]
+                container_name = container['Name'].lstrip('/')
+                name = '/docker/' + container_name
+                cache[container_id] = name
+                return name
+        except Exception:
+            pass
+
+    cache[container_id] = default
+    return default
+
+
 def to_human(num, suffix='B'):
     num = int(num)
     for unit in [' ','K','M','G','T','P','E','Z']:
@@ -162,12 +230,7 @@ def run(user, cmd, interactive=False):
         pty.spawn(prefix+cmd)
 
         # Restore screen
-        curses.start_color() # load colors
-        curses.use_default_colors()
-        curses.noecho()      # do not echo text
-        curses.cbreak()      # do not wait for "enter"
-        curses.curs_set(0)   # hide cursor
-        curses.mousemask(curses.ALL_MOUSE_EVENTS)
+        init_screen()
         curses.resetty()
     else:
         with open('/dev/null', 'w') as dev_null:
@@ -184,9 +247,19 @@ class Cgroup(object):
         self.base_path = base_path
 
     @property
-    def name(self):
+    def short_path(self):
         return self.path[len(self.base_path):] or '/'
 
+    @property
+    def name(self):
+        if HAS_DOCKER and self.type == 'docker':
+            container_id = self.short_path
+            for prefix in DOCKER_PREFIXES:
+                container_id = strip_prefix(prefix, container_id)
+            return docker_container_name(container_id, default=self.short_path)
+
+        return self.short_path
+
     @property
     def owner(self):
         path = os.path.join(self.base_path, self.path, 'tasks')
@@ -198,10 +271,10 @@ class Cgroup(object):
 
     @property
     def type(self):
-        path = self.name
+        path = self.short_path
 
         # Guess cgroup owner
-        if path.startswith('/docker/') or path.startswith('/system.slice/docker-'):
+        if any(path.startswith(prefix) for prefix in DOCKER_PREFIXES):
             return 'docker'
         elif path.startswith('/lxc/'):
             return 'lxc'
@@ -213,6 +286,8 @@ class Cgroup(object):
             return 'systemd'
         elif path == '/user.slice' or path == '/system.slice' or path.startswith('/system.slice/'):
             return 'systemd'
+        elif regexp_ovz_container.match(path) and path != '/0' and HAS_OPENVZ:
+            return 'openvz'
         else:
             return '-'
 
@@ -239,11 +314,11 @@ class Cgroup(object):
             content = content.split('\n')
 
             if ' ' in content[0]:
-                content = dict((re.split(' +', l, 1) for l in content))
+                content = dict((re.split(' +', l, 1) for l in content if content))
                 for k, v in content.items():
                     content[k] = self._coerce(v)
             else:
-                content = [self._coerce(v) for v in content]
+                content = [self._coerce(v) for v in content if v]
 
         else:
             content = self._coerce(content)
@@ -283,34 +358,43 @@ def init():
             if arg in subsystems and arg not in CGROUP_MOUNTPOINTS:
                 CGROUP_MOUNTPOINTS[arg] = mount[1]
 
+def collect_ensure_common(data, cgroup):
+    '''
+    Some cgroup exists in only one controller. Attempt to collect common metrics
+    (tasks clount, owner, ...) from the first controller we find the task in.
+    '''
+    if 'tasks' in data:
+        return
+
+    # Collect
+    data['tasks'] = cgroup['tasks']
+    data['owner'] = cgroup.owner
+    data['type'] = cgroup.type
+
+def get_user_beacounts():
+    '''
+    get memory stats(via privvmpages) from vzlist output to openvz containers
+    '''
+    prefix = []
+    if os.getuid() != 0:
+       prefix = ['sudo']
+
+    command = prefix + ['vzlist', '-o', 'ctid,privvmpages,privvmpages.l', '-H']
+    output, err = subprocess.Popen(command, shell=False, stdout=subprocess.PIPE).communicate()
+    return output
+
+
 def collect(measures):
     cur = defaultdict(dict)
     prev = measures['data']
 
-    # Collect global data
-    if 'cpuacct' in CGROUP_MOUNTPOINTS:
-        # list all "folders" under mountpoint
-        for cgroup in cgroups(CGROUP_MOUNTPOINTS['cpuacct']):
-            # Collect tasks
-            cur[cgroup.name]['tasks'] = cgroup['tasks']
-
-            # Collect user
-            cur[cgroup.name]['owner'] = cgroup.owner
-
-            # Collect cgroup type
-            cur[cgroup.name]['type'] = cgroup.type
-
-    # Collect memory statistics
-    if 'memory' in CGROUP_MOUNTPOINTS:
-        # list all "folders" under mountpoint
-        for cgroup in cgroups(CGROUP_MOUNTPOINTS['memory']):
-            cur[cgroup.name]['memory.usage_in_bytes'] = cgroup['memory.usage_in_bytes']
-            cur[cgroup.name]['memory.limit_in_bytes'] = min(int(cgroup['memory.limit_in_bytes']), measures['global']['total_memory'])
 
     # Collect CPU statistics
     if 'cpuacct' in CGROUP_MOUNTPOINTS:
         # list all "folders" under mountpoint
         for cgroup in cgroups(CGROUP_MOUNTPOINTS['cpuacct']):
+            collect_ensure_common(cur[cgroup.name], cgroup)
+
             # Collect CPU stats
             cur[cgroup.name]['cpuacct.stat'] = cgroup['cpuacct.stat']
             cur[cgroup.name]['cpuacct.stat.diff'] = {'user':0, 'system':0}
@@ -324,9 +408,17 @@ def collect(measures):
     if 'blkio' in CGROUP_MOUNTPOINTS:
         # list all "folders" under mountpoint
         for cgroup in cgroups(CGROUP_MOUNTPOINTS['blkio']):
+            collect_ensure_common(cur[cgroup.name], cgroup)
+
             # Collect BlockIO stats
-            cur[cgroup.name]['blkio.throttle.io_service_bytes'] = cgroup['blkio.throttle.io_service_bytes']
-            cur[cgroup.name]['blkio.throttle.io_service_bytes.diff'] = {'total':0}
+            try:
+                cur[cgroup.name]['blkio.throttle.io_service_bytes'] = cgroup['blkio.throttle.io_service_bytes']
+                cur[cgroup.name]['blkio.throttle.io_service_bytes.diff'] = {'total':0}
+            except IOError as e:
+                # Workaround broken systems (see #15)
+                if e.errno == errno.ENOENT:
+                    continue
+                raise
 
             # Collect BlockIO increase on run > 1
             if cgroup.name in prev:
@@ -334,6 +426,50 @@ def collect(measures):
                 prev_val = prev[cgroup.name]['blkio.throttle.io_service_bytes']['Total']
                 cur[cgroup.name]['blkio.throttle.io_service_bytes.diff']['total'] = cur_val - prev_val
 
+    # Collect memory statistics
+    if 'memory' in CGROUP_MOUNTPOINTS:
+        # list all "folders" under mountpoint
+        for cgroup in cgroups(CGROUP_MOUNTPOINTS['memory']):
+            collect_ensure_common(cur[cgroup.name], cgroup)
+            cur[cgroup.name]['memory.usage_in_bytes'] = cgroup['memory.usage_in_bytes']
+            cur[cgroup.name]['memory.limit_in_bytes'] = min(int(cgroup['memory.limit_in_bytes']), measures['global']['total_memory'])
+
+    # Collect PIDs constraints. Root cgroup does *not* have the controller files
+    if 'pids' in CGROUP_MOUNTPOINTS:
+        # list all "folders" under mountpoint
+        for cgroup in cgroups(CGROUP_MOUNTPOINTS['pids']):
+            if cgroup.name == "/":
+                continue
+            collect_ensure_common(cur[cgroup.name], cgroup)
+            cur[cgroup.name]['pids.max'] = cgroup['pids.max']
+
+    #Collect memory statistics for openvz
+    if HAS_OPENVZ:
+        user_beancounters = get_user_beacounts()
+        # We have lines like -
+        #      1202     202419    2457600
+        #      1203     299835    2457600
+        #      1207      54684    2457600
+        #      1210     304939    2457600
+        #1000001212      13493    2457600
+        for line in user_beancounters.split('\n'):
+            if line == '':
+                continue
+            line = re.sub(r'^\s+', '', line)
+            splited_line = re.split('\s+', line)
+            if len(splited_line) != 3:
+                continue
+            ctid, privvmpages, limit = splited_line
+            ctid = '/' + ctid
+            if 'tasks' not in cur[ctid]:
+                continue
+            privvmpages = int(privvmpages)
+            privvmpages = privvmpages * 4096
+            limit = int(limit)
+            limit = limit * 4096
+            cur[ctid]['memory.usage_in_bytes'] = privvmpages
+            cur[ctid]['memory.limit_in_bytes'] = min(limit, measures['global']['total_memory'])
+
     # Sanity check: any data at all ?
     if not len(cur):
         raise KeyboardInterrupt()
@@ -356,7 +492,8 @@ def built_statistics(measures, conf):
         line = {
             'owner': str(data.get('owner', 'nobody')),
             'type': str(data.get('type', 'cgroup')),
-            'tasks': len(data['tasks']),
+            'cur_tasks': len(data['tasks']),
+            'max_tasks': data.get('pids.max', 'max'),
             'memory_cur_bytes': data.get('memory.usage_in_bytes', 0),
             'memory_limit_bytes': data.get('memory.limit_in_bytes', measures['global']['total_memory']),
             'cpu_total_seconds': data.get('cpuacct.stat', {}).get('system', 0) + data.get('cpuacct.stat', {}).get('user', 0),
@@ -369,6 +506,7 @@ def built_statistics(measures, conf):
         line['cpu_total_str'] = to_human_time(line['cpu_total_seconds'])
         line['memory_cur_percent'] = line['memory_cur_bytes'] / line['memory_limit_bytes']
         line['memory_cur_str'] = "{0: >7}/{1: <7}".format(to_human(line['memory_cur_bytes']), to_human(line['memory_limit_bytes']))
+        line['tasks'] = "{0: >5}/{1: <5}".format(line['cur_tasks'], line['max_tasks'])
         line['blkio_bw'] = to_human(line['blkio_bw_bytes'], 'B/s')
         results.append(line)
 
@@ -383,7 +521,6 @@ def render_tree(results, tree, level=0, prefix=[], node='/'):
     for i, line in enumerate(tree[node]):
         cgroup = line['cgroup']
 
-
         # Build name
         if i == len(tree[node]) - 1:
             line['_tree'] = prefix + [curses.ACS_LLCORNER, curses.ACS_HLINE, ' ']
@@ -399,10 +536,46 @@ def render_tree(results, tree, level=0, prefix=[], node='/'):
         else:
             line['_tree'] [-2] = '+'
 
+def filter_tree(tree, keep, node='/'):
+    '''
+    Keep a branch if and only if it is of of a 'keep' type or has a child of
+    the 'keep' type
+    '''
+    filtered = []
+
+    # Filter
+    for cgroup in tree.get(node, []):
+        if filter_tree(tree, keep, cgroup['cgroup']):
+            filtered.append(cgroup)
+        elif cgroup['type'] in keep:
+            filtered.append(cgroup)
+
+    # Commit
+    if filtered:
+        tree[node] = filtered
+    else:
+        tree.pop(node, None)
+
+    return bool(filtered)
+
 def prepare_tree(results):
-    # Build tree
+    '''
+    Filter results for matching types and render tree
+    '''
+    ## List view
+    if not CONFIGURATION['tree']:
+        # Fast track: if there is no filter, do not filter
+        if not CONFIGURATION['type']:
+            return results
+
+        # Slow track: do filter
+        return [l for l in results if l['type'] in CONFIGURATION['type']]
+
+    ## Tree view
     tree = {}
     rendered = []
+
+    # Build tree
     for line in results:
         cgroup = line['cgroup']
         parent = os.path.dirname(cgroup)
@@ -417,16 +590,18 @@ def prepare_tree(results):
             tree[parent] = []
         tree[parent].append(line)
 
+    # If there are filters, filter
+    if CONFIGURATION['type']:
+         filter_tree(tree, CONFIGURATION['type'])
+
     # Render tree, starting from root
     render_tree(rendered, tree)
     return rendered
 
 def display(scr, results, conf):
-    # Sort
+    # Sort and render
     results = sorted(results, key=lambda line: line.get(conf['sort_by'], 0), reverse=not conf['sort_asc'])
-
-    if CONFIGURATION['tree']:
-        results = prepare_tree(results)
+    results = prepare_tree(results)
 
     CONFIGURATION['cgroups'] = [cgroup['cgroup'] for cgroup in results]
 
@@ -444,9 +619,24 @@ def display(scr, results, conf):
         CONFIGURATION['selected_line_name'] = CONFIGURATION['cgroups'][CONFIGURATION['selected_line_num']]
     CONFIGURATION['selected_line'] = results[CONFIGURATION['selected_line_num']]
 
+    # Get display informations
+    height, width = scr.getmaxyx()
+    list_height = height - 2 # title + status lines
+
+    # Update offset
+    max_offset = max(0, len(results) - list_height)
+    # selected line above screen limit
+    if CONFIGURATION['selected_line_num'] < CONFIGURATION['offset']:
+        CONFIGURATION['offset'] = CONFIGURATION['selected_line_num']
+    # selected line below screen limit
+    elif CONFIGURATION['selected_line_num'] - CONFIGURATION['offset'] > list_height - 1:
+        CONFIGURATION['offset'] = CONFIGURATION['selected_line_num'] - list_height + 1
+    # offset non consistent
+    elif CONFIGURATION['offset'] > max_offset:
+        CONFIGURATION['offset'] = max_offset
+
     # Display statistics
     scr.clear()
-    height, width = scr.getmaxyx()
 
     # Title line && templates
     x = 0
@@ -470,9 +660,9 @@ def display(scr, results, conf):
 
     # Content
     lineno = 1
-    for line in results:
+    for line in results[CONFIGURATION['offset']:]:
         y = 0
-        if lineno-1 == CONFIGURATION['selected_line_num']:
+        if lineno-1 == CONFIGURATION['selected_line_num']-CONFIGURATION['offset']:
             col_reg, col_tree = curses.color_pair(2), curses.color_pair(2)
         else:
             col_reg, col_tree = colors = curses.color_pair(0), curses.color_pair(4)
@@ -480,7 +670,7 @@ def display(scr, results, conf):
         # Draw line background
         try:
             scr.addstr(lineno, 0, ' '*width, col_reg)
-        except:
+        except _curses.error:
             # Handle small screens
             break
 
@@ -500,14 +690,14 @@ def display(scr, results, conf):
                 scr.addstr(lineno, y, cell_tpl.format(data_point)+' ', col_reg)
                 if col.width:
                     y += col.width + 1
-        except:
+        except _curses.error:
             # Handle narrow screens
             pass
         lineno += 1
     else:
         # Make sure last line did not wrap, clear it if needed
         try: scr.addstr(lineno, 0, ' '*width)
-        except: pass
+        except _curses.error: pass
 
     # status line
     try:
@@ -537,12 +727,16 @@ def display(scr, results, conf):
         # Do we have any actions available for *selected* line ?
         selected_type = selected['type']
         if selected_type == 'docker' and HAS_DOCKER or \
-           selected_type in ['lxc', 'lxc-user'] and HAS_LXC:
-            scr.addstr(" [A]ttach, [E]nter, [S]top, [K]ill ", color)
-            scr.addch(curses.ACS_VLINE, color)
+           selected_type in ['lxc', 'lxc-user'] and HAS_LXC or \
+           selected_type == 'openvz' and HAS_OPENVZ:
+             if selected_type == 'openvz':
+                scr.addstr(" [A]ttach, [E]nter, [S]top, [C]hkpnt, [K]ill ", color)
+             else:
+                scr.addstr(" [A]ttach, [E]nter, [S]top, [K]ill ", color)
+             scr.addch(curses.ACS_VLINE, color)
 
         scr.addstr(" [Q]uit", color)
-    except:
+    except _curses.error:
         # Handle narrow screens
         pass
 
@@ -580,6 +774,8 @@ def on_keyboard(c):
             run(-2, ['docker', 'attach', selected_name], interactive=True)
         elif selected['type'] in ['lxc', 'lxc-user'] and HAS_LXC:
             run(selected['owner'], ['lxc-console', '--name', selected_name, '--', '/bin/bash'], interactive=True)
+        elif selected['type'] == 'openvz' and HAS_OPENVZ:
+            run(selected['owner'], ['vzctl', 'console', selected_name], interactive=True)
 
         return 2
     elif c == ord('e'):
@@ -592,6 +788,8 @@ def on_keyboard(c):
             run(-2, ['docker', 'exec', '-it', selected_name, '/bin/bash'], interactive=True)
         elif selected['type'] in ['lxc', 'lxc-user'] and HAS_LXC:
             run(selected['owner'], ['lxc-attach', '--name', selected_name, '--', '/bin/bash'], interactive=True)
+        elif selected['type'] == 'openvz' and HAS_OPENVZ:
+            run(selected['owner'], ['vzctl', 'enter', selected_name], interactive=True)
 
         return 2
     elif c == ord('s'):
@@ -604,6 +802,17 @@ def on_keyboard(c):
             run(-2, ['docker', 'stop', selected_name])
         elif selected['type'] in ['lxc', 'lxc-user'] and HAS_LXC:
             run(selected['owner'], ['lxc-stop', '--name', selected_name, '--nokill', '--nowait'])
+        elif selected['type'] == 'openvz' and HAS_OPENVZ:
+            run(selected['owner'], ['vzctl', 'stop', selected_name])
+
+        return 1
+    elif c == ord('c'):
+        selected = CONFIGURATION['selected_line']
+        selected_name = os.path.basename(selected['cgroup'])
+
+        if selected['type'] == 'openvz' and HAS_OPENVZ:
+            run(selected['owner'], ['vzctl', 'chkpnt', selected_name])
+
         return 1
     elif c == ord('k'):
         selected = CONFIGURATION['selected_line']
@@ -615,6 +824,9 @@ def on_keyboard(c):
             run(-2, ['docker', 'stop', '-t', '0', selected_name])
         elif selected['type'] in ['lxc', 'lxc-user'] and HAS_LXC:
             run(selected['owner'], ['lxc-stop', '-k', '--name', selected_name, '--nowait'])
+        elif selected['type'] == 'openvz' and HAS_OPENVZ:
+            run(selected['owner'], ['vzctl', 'stop', selected_name, '--fast'])
+
         return 2
     elif c == 269: # F5
         CONFIGURATION['tree'] = not CONFIGURATION['tree']
@@ -669,7 +881,7 @@ def on_mouse():
 
 def on_resize():
     '''Redraw screen, do not refresh'''
-    return 1
+    return 2
 
 def event_listener(scr, timeout):
     '''
@@ -715,6 +927,20 @@ Hint: It seems you have 'boot2docker' installed.
       $ docker run --volume=/sys/fs/cgroup:/sys/fs/cgroup:ro -it --rm yadutaf/ctop""", file=sys.stderr)
     devnull.close()
 
+def init_screen():
+    curses.start_color() # load colors
+    curses.use_default_colors()
+    curses.noecho()      # do not echo text
+    curses.cbreak()      # do not wait for "enter"
+    curses.mousemask(curses.ALL_MOUSE_EVENTS)
+
+    # Hide cursor, if terminal AND curse supports it
+    if hasattr(curses, 'curs_set'):
+        try:
+            curses.curs_set(0)
+        except:
+            pass
+
 def main():
     # Parse arguments
     parser = OptionParser()
@@ -722,6 +948,7 @@ def main():
     parser.add_option("--refresh",  action="store",      type="int",    default=1,     help="Refresh display every <seconds>")
     parser.add_option("--follow",   action="store",      type="string", default="",    help="Follow cgroup path")
     parser.add_option("--fold",     action="append",                                   help="Fold cgroup sub tree")
+    parser.add_option("--type",     action="append",                                   help="Only show containers of this type")
     parser.add_option("--columns",  action="store",      type="string", default="owner,type,processes,memory,cpu-sys,cpu-user,blkio,cpu-time", help="List of optional columns to display. Always includes 'name'")
     parser.add_option("--sort-col", action="store",      type="string", default="cpu-user", help="Select column to sort by initially. Can be changed dynamically.")
 
@@ -731,6 +958,7 @@ def main():
     CONFIGURATION['refresh_interval'] = float(options.refresh)
     CONFIGURATION['columns'] = []
     CONFIGURATION['fold'] = options.fold or list()
+    CONFIGURATION['type'] = options.type or list()
 
     if options.follow:
         CONFIGURATION['selected_line_name'] = options.follow
@@ -775,13 +1003,8 @@ def main():
     try:
         # Curse initialization
         stdscr = curses.initscr()
-        curses.start_color() # load colors
-        curses.use_default_colors()
-        curses.noecho()      # do not echo text
-        curses.cbreak()      # do not wait for "enter"
-        curses.curs_set(0)   # hide cursor
-        stdscr.keypad(1)     # parse keypad controll sequences
-        curses.mousemask(curses.ALL_MOUSE_EVENTS)
+        init_screen()
+        stdscr.keypad(1)     # parse keypad control sequences
 
         # Curses colors
         curses.init_pair(1, curses.COLOR_BLACK, curses.COLOR_GREEN) # header
